@@ -1,5 +1,5 @@
 package DBIx::Lite::ResultSet;
-$DBIx::Lite::ResultSet::VERSION = '0.21';
+$DBIx::Lite::ResultSet::VERSION = '0.22';
 use strict;
 use warnings;
 
@@ -28,7 +28,7 @@ sub _new {
     
     # optional arguments
     for (grep exists($params{$_}), qw(joins where select group_by having order_by
-        limit offset rows_per_page page cur_table)) {
+        limit offset for_update rows_per_page page cur_table)) {
         $self->{$_} = delete $params{$_};
     }
     $self->{cur_table} //= $self->{table};
@@ -58,6 +58,14 @@ for my $methname (qw(group_by having order_by limit offset rows_per_page page))
     };
 }
 
+sub for_update {
+    my ($self) = @_;
+    
+    my $new_self = $self->_clone;
+    $new_self->{for_update} = 1;
+    $new_self;
+}
+
 # return a clone of this object
 sub _clone {
     my $self = shift;
@@ -190,7 +198,7 @@ sub select_sql {
             unless ref $self->{order_by} eq 'ARRAY';
     }
     
-    return $self->{dbix_lite}->{abstract}->select(
+    my ($sql, @bind) = $self->{dbix_lite}->{abstract}->select(
         -columns    => [ uniq @cols ],
         -from       => [ -join => $self->{table}{name} . "|me", @joins ],
         -where      => { -and => $self->{where} },
@@ -200,6 +208,12 @@ sub select_sql {
         $self->{limit}      ? (-limit       => $self->{limit})      : (),
         $self->{offset}     ? (-offset      => $self->{offset})     : (),
     );
+    
+    if ($self->{for_update}) {
+        $sql .= " FOR UPDATE";
+    }
+    
+    return ($sql, @bind);
 }
 
 sub select_sth {
@@ -446,7 +460,8 @@ sub count {
     
     my $count;
     $self->{dbix_lite}->dbh_do(sub {
-        my $count_rs = ($self->_clone)->select(\ "-COUNT(*)");
+        # Postgres throws an error when using ORDER BY clauses with COUNT(*)
+        my $count_rs = $self->select(\ "-COUNT(*)")->order_by(undef);
         my ($sth, @bind) = $count_rs->select_sth;
         $sth->execute(@bind);
         $count = +($sth->fetchrow_array)[0];
@@ -625,7 +640,7 @@ DBIx::Lite::ResultSet
 
 =head1 VERSION
 
-version 0.21
+version 0.22
 
 =head1 OVERVIEW
 
@@ -728,6 +743,21 @@ It returns a L<DBIx::Lite::ResultSet> object to allow for further method chainin
 
     my $rs = $books_rs->limit(5)->offset(10);
 
+=head2 for_update
+
+This method accepts no argument. It enables the addition of the SQL C<FOR UPDATE>
+clause at the end of the query, which allows to fetch data and lock it for updating.
+It returns a L<DBIx::Lite::ResultSet> object to allow for further method chaining.
+Note that no records are actually locked until the query is executed with L<single()>,
+L<all()> or L<next()>.
+
+    $dbix->txn(sub {
+        my $author = $dbix->table('authors')->find($id)->for_update->single
+            or die "Author not found";
+        
+        $author->update({ age => 30 });
+    });
+
 =head2 inner_join
 
 This method accepts the name of a column to join and a set of join conditions.
